package com.lantone.security.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.dto.DataAuthDetailDTO;
import com.lantone.common.dto.DeptDTO;
import com.lantone.common.dto.GetDataAuthByIdDTO;
import com.lantone.common.dto.GetDataAuthPageDTO;
import com.lantone.common.dto.GetDoctorPageDTO;
import com.lantone.common.dto.HospitalDTO;
import com.lantone.common.dto.RoleDTO;
import com.lantone.common.dto.SoftwareRoleDataAuthDetailDTO;
import com.lantone.common.dto.SysServiceDTO;
import com.lantone.common.exception.Asserts;
import com.lantone.common.util.DateUtil;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.SysUserUtils;
import com.lantone.common.vo.AddAuthSoftwareVO;
import com.lantone.common.vo.AddAuthVO;
import com.lantone.common.vo.GetDataAuthPageVO;
import com.lantone.common.vo.GetDoctorPageVO;
import com.lantone.common.vo.UpdateAuthVO;
import com.lantone.dblayermbg.entity.DataAuth;
import com.lantone.dblayermbg.entity.DataAuthDetail;
import com.lantone.dblayermbg.entity.Dept;
import com.lantone.dblayermbg.entity.Hospital;
import com.lantone.dblayermbg.entity.HospitalUser;
import com.lantone.dblayermbg.entity.Menu;
import com.lantone.dblayermbg.entity.RoleSoftwareDataAuth;
import com.lantone.dblayermbg.entity.SoftwareDataAuth;
import com.lantone.dblayermbg.facade.DataAuthDetailFacade;
import com.lantone.dblayermbg.facade.DataAuthFacade;
import com.lantone.dblayermbg.facade.DeptFacade;
import com.lantone.dblayermbg.facade.DoctorFacade;
import com.lantone.dblayermbg.facade.HospitalFacade;
import com.lantone.dblayermbg.facade.HospitalUserFacade;
import com.lantone.dblayermbg.facade.RoleSoftwareDataAuthFacade;
import com.lantone.dblayermbg.facade.SoftwareDataAuthFacade;
import com.lantone.dblayermbg.service.impl.DataAuthDetailServiceImpl;
import com.lantone.dblayermbg.service.impl.RoleSoftwareDataAuthServiceImpl;
import com.lantone.security.enums.DataAuthDataTypeEnum;
import com.lantone.security.enums.DataAuthDetailTypeEnum;
import com.lantone.security.enums.HospitalTypeEnum;
import com.lantone.common.enums.IsDeleteEnum;
import com.lantone.security.enums.RelationEnum;
import com.lantone.security.enums.ServiceEnum;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;


/**
 * @Description:数据权限操作facade
 * @Author songxl
 * @Date 2021/7/21
 */
@Component
public class DataAuthManagementFacade {
    @Autowired
    private DataAuthFacade dataAuthFacade;
    @Autowired
    private SoftwareDataAuthFacade softwareDataAuthFacade;
    @Autowired
    private RoleSoftwareDataAuthFacade roleSoftwareDataAuthFacade;
    @Autowired
    private DataAuthDetailFacade dataAuthDetailFacade;
    @Autowired
    private HospitalFacade hospitalFacade;
    @Autowired
    private DeptFacade deptFacade;
    @Autowired
    private DoctorFacade doctorFacade;
    @Autowired
    private HospitalUserFacade hospitalUserFacade;


    /**
     * @param Id
     * @param status
     * @Description禁用启用数据权限
     * @Return boolean
     */
    public boolean disableDataAuth(Long Id, String status) {
        return dataAuthFacade.update(new UpdateWrapper<DataAuth>()
                .set("status", status)
                .eq("id", Id)
                .eq("is_deleted", IsDeleteEnum.N.getKey()));
    }

    /**
     * @Description修改数据权限
     * @Param [softwareDataAuthVO]
     * @Return boolean
     */
    public boolean updateDataAuth(UpdateAuthVO updateAuthVO) {

        //1.修改数据权限
        boolean dataAuthUpdate = dataAuthFacade.update(new UpdateWrapper<DataAuth>()
                .set("name", updateAuthVO.getUpdateDataAuthVO().getName())
                .eq("id", updateAuthVO.getUpdateDataAuthVO().getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey()));
        if (dataAuthUpdate) {
            //2.获取该权限 系统与数据权限信息关联表（sys_service_data_auth）的关联id
            List<Long> serviceDataAuthIDS = softwareDataAuthFacade.list(new QueryWrapper<SoftwareDataAuth>()
                    .eq("data_auth_id", updateAuthVO.getUpdateDataAuthVO().getId())
                    .eq("is_deleted", IsDeleteEnum.N.getKey())).stream().map(SoftwareDataAuth::getId).collect(Collectors.toList());
            //3.通过serviceDataAuthIDS 删除sys_role_service_data_auth和sys_service_data_auth和sys_data_auth_detail 对应关系
            if (ListUtil.isNotEmpty(serviceDataAuthIDS)) {
                if (!deleteServiceDataAuth(serviceDataAuthIDS)) {
                    Asserts.fail("数据权限与系统关系删除失败");
                }
            }
            //4.插入新建数据权限与系统的关系
            return saveServiceDataAuth(updateAuthVO.getUpdateDataAuthVO().getSoftwareVOS(), updateAuthVO.getRoles(), updateAuthVO.getUpdateDataAuthVO().getId());
        } else {
            Asserts.fail("数据权限修改失败");
        }
        return false;
    }

    /**
     * @Description通过serviceDataAuthIDS删除sys_role_service_data_auth和sys_service_data_auth和sys_data_auth_detail对应关系
     * @Param [serviceDataAuthIDS]
     * @Return void
     */
    public boolean deleteServiceDataAuth(List<Long> serviceDataAuthIDS) {
        if (dataAuthDetailFacade.remove(new UpdateWrapper<DataAuthDetail>()
                .in("software_data_auth_id", serviceDataAuthIDS)
                .eq("is_deleted", IsDeleteEnum.N.getKey()))) {
            if (roleSoftwareDataAuthFacade.remove(new UpdateWrapper<RoleSoftwareDataAuth>()
                    .in("software_data_auth_id", serviceDataAuthIDS)
                    .eq("is_deleted", IsDeleteEnum.N.getKey()))) {
                return softwareDataAuthFacade.remove(new UpdateWrapper<SoftwareDataAuth>()
                        .in("id", serviceDataAuthIDS)
                        .eq("is_deleted", IsDeleteEnum.N.getKey()));
            } else {
                Asserts.fail("数据权限与系统、角色关联关系删除失败");
            }
        } else {
            Asserts.fail("系统数据权限与数据权限明细关联关系删除失败");
        }
        return false;
    }

    /**
     * @param Id
     * @Description删除数据权限
     * @Return boolean
     */
    public boolean deleteDataAuth(Long Id) {
        //1.删除数据权限
        boolean dataAuthdelete = dataAuthFacade.update(new UpdateWrapper<DataAuth>()
                .set("is_deleted", IsDeleteEnum.Y.getKey())
                .eq("id", Id));
        if (dataAuthdelete) {
//            //2.获取该权限 系统与数据权限信息关联表（sys_service_data_auth）的关联id
//            List<Long> serviceDataAuthIDS = softwareDataAuthFacade.list(new QueryWrapper<SoftwareDataAuth>()
//                    .eq("data_auth_id", Id)
//                    .eq("is_deleted", IsDeleteEnum.N.getKey())).stream().map(SoftwareDataAuth::getId).collect(Collectors.toList());
//            //3.通过serviceDataAuthIDS 删除sys_role_service_data_auth和sys_service_data_auth和sys_data_auth_detail 对应关系
//            if (ListUtil.isNotEmpty(serviceDataAuthIDS)) {
//                if (deleteServiceDataAuth(serviceDataAuthIDS)) {
//                    return true;
//                } else {
//                    Asserts.fail("数据权限与系统关系删除失败");
//                }
//            }
            return true;
        } else {
            Asserts.fail("数据权限删除失败");
        }
        return false;
    }

    /**
     * @param addAuthVO
     * @Description添加数据权限
     * @Return boolean
     */
    public boolean addDataAuth(AddAuthVO addAuthVO) {
        //1.插入新建数据权限
        DataAuth dataAuth = new DataAuth();
        BeanUtils.copyProperties(addAuthVO.getAddDataAuthVO(), dataAuth);
        dataAuth.setGmtCreate(DateUtil.now());
        dataAuth.setCreator(SysUserUtils.getCurrentPrincipleId()+"");
        if (dataAuthFacade.save(dataAuth)) {
            //2.插入新建数据权限与系统的关系 sys_software_data_auth
            return saveServiceDataAuth(addAuthVO.getAddDataAuthVO().getSoftwareVOS(), addAuthVO.getRoles(), dataAuth.getId());
        } else {
            Asserts.fail("数据权限插入失败");
        }
        return false;

    }

    /**
     * @param softwareVOS
     * @param roles
     * @param dataAuthId
     * @Description插入新建数据权限与系统的关系
     * @Return void
     */
    private boolean saveServiceDataAuth(List<AddAuthSoftwareVO> softwareVOS, List<Long> roles, Long dataAuthId) {
        AtomicBoolean out = new AtomicBoolean(false);
        softwareVOS.stream().forEach(softwareVO->{
            SoftwareDataAuth softwareDataAuth = new SoftwareDataAuth();
            softwareDataAuth.setSoftwareId(softwareVO.getId());
            softwareDataAuth.setDataAuthId(dataAuthId);
            softwareDataAuth.setGmtCreate(DateUtil.now());
            softwareDataAuth.setCreator(SysUserUtils.getCurrentPrincipleId()+"");
            if (softwareDataAuthFacade.save(softwareDataAuth)) {
                //2.插入数据权限系统关联表与角色管理sys_role_service_data_auth
                List<RoleSoftwareDataAuth> roleSoftwareDataAuths = new ArrayList<>();
                for (Long roleId : roles) {
                    RoleSoftwareDataAuth roleServiceDataAuth = new RoleSoftwareDataAuth();
                    roleServiceDataAuth.setRoleId(roleId);
                    roleServiceDataAuth.setSoftwareDataAuthId(softwareDataAuth.getId());
                    roleServiceDataAuth.setGmtCreate(DateUtil.now());
                    roleServiceDataAuth.setCreator(SysUserUtils.getCurrentPrincipleId()+"");
                    roleSoftwareDataAuths.add(roleServiceDataAuth);
                }
                if (new RoleSoftwareDataAuthServiceImpl().saveBatch(roleSoftwareDataAuths)) {
                    //3.插入数据权限详细信息。sys_data_auth_detail
                    //自定义时
                    List<DataAuthDetail> dataAuthDetails = new ArrayList<>();
                    if (ListUtil.isNotEmpty(softwareVO.getDataAuthDetail().getDetailIds())
                            && softwareVO.getDataAuthDetail().getDetailType() != null) {
                        softwareVO.getDataAuthDetail().getDetailIds().stream().forEach(id -> {
                            DataAuthDetail dataAuthDetail = new DataAuthDetail();
                            BeanUtils.copyProperties(softwareVO.getDataAuthDetail(), dataAuthDetail);
                            dataAuthDetail.setGmtCreate(DateUtil.now());
                            dataAuthDetail.setCreator(SysUserUtils.getCurrentPrincipleId() + "");
                            dataAuthDetail.setSoftwareDataAuthId(softwareDataAuth.getId());
                            dataAuthDetail.setDetailId(id);
                            dataAuthDetails.add(dataAuthDetail);
                        });
                    } else {
                        DataAuthDetail dataAuthDetail = new DataAuthDetail();
                        BeanUtils.copyProperties(softwareVO.getDataAuthDetail(), dataAuthDetail);
                        dataAuthDetail.setGmtCreate(DateUtil.now());
                        dataAuthDetail.setCreator(SysUserUtils.getCurrentPrincipleId() + "");
                        dataAuthDetail.setSoftwareDataAuthId(softwareDataAuth.getId());
                        dataAuthDetails.add(dataAuthDetail);
                    }
                    if(new DataAuthDetailServiceImpl().saveBatch(dataAuthDetails)){
                        out.set(true);
                    }else {
                        Asserts.fail("数据权限详情插入失败");
                    }
                } else {
                    Asserts.fail("角色对应服务数据权限关系插入失败");
                }

            } else {
                Asserts.fail("服务对应数据权限关系插入失败");
            }
        });
        return out.get();
    }

    /**
     * @Description通过数据权限id获取数据权限详情
     * @Param [serviceDataAuthVO]
     * @Return com.lantone.security.dto.DataAuthDTO
     */
    public GetDataAuthByIdDTO getDataAuthById(Long Id) {
        GetDataAuthByIdDTO dataAuthDTO = new GetDataAuthByIdDTO();
        List<SoftwareRoleDataAuthDetailDTO> serviceRoleDataAuthDetailDTOS = dataAuthFacade.getBaseMapper().getDataAuthById(Id);
        if (ListUtil.isNotEmpty(serviceRoleDataAuthDetailDTOS)) {
            //1.添加数据权限信息
            dataAuthDTO.setId(serviceRoleDataAuthDetailDTOS.get(0).getDataAuthId());
            dataAuthDTO.setName(serviceRoleDataAuthDetailDTOS.get(0).getDataAuthName());
            dataAuthDTO.setStatus(serviceRoleDataAuthDetailDTOS.get(0).getStatus());
            //2.获取系统信息
            Set<Long> serviceSet = serviceRoleDataAuthDetailDTOS.stream().map(SoftwareRoleDataAuthDetailDTO::getSoftwareId).collect(Collectors.toSet());
            List<SysServiceDTO> sysServiceDTOS = new ArrayList<>();
            serviceSet.stream().forEach(id -> {
                //3.添加服务信息
                SysServiceDTO sysServiceDTO = new SysServiceDTO();
                sysServiceDTO.setId(id);
                sysServiceDTO.setName(ServiceEnum.getName(id.intValue()));
                //4.1添加数据权限详情列表
                //                List<ServiceRoleDataAuthDetailDTO> oneServiceRoleDataAuthDetailDTOS = serviceRoleDataAuthDetailDTOS.stream()
                //                        .filter(serviceRoleDataAuthDetailDTO -> serviceRoleDataAuthDetailDTO.getServiceId() == id).collect(Collectors.toList());
                List<SoftwareRoleDataAuthDetailDTO> oneServiceRoleDataAuthDetailDTOS = dataAuthFacade.getBaseMapper().getOneSoftwareRoleDataAuthDetail(id, Id);

                List<DataAuthDetailDTO> dataAuthDetailDTOS = new ArrayList<>();
                //4.2.1是否获取当前组织-科室信息 dataAuthDetailDTOS去重
                oneServiceRoleDataAuthDetailDTOS.stream().forEach(serviceRoleDataAuthDetailDTO -> {
                    DataAuthDetailDTO dataAuthDetailDTO = new DataAuthDetailDTO();
                    dataAuthDetailDTO.setId(serviceRoleDataAuthDetailDTO.getDetail());
                    dataAuthDetailDTO.setDataType(serviceRoleDataAuthDetailDTO.getDataType());
                    dataAuthDetailDTO.setDetailType(serviceRoleDataAuthDetailDTO.getDetailType());
                    dataAuthDetailDTO.setDetailId(serviceRoleDataAuthDetailDTO.getDetailId());
                    if (DataAuthDataTypeEnum.CUSTOMIZE.getKey() == serviceRoleDataAuthDetailDTO.getDataType()
                            && serviceRoleDataAuthDetailDTO.getDetailType() != null
                            && DataAuthDetailTypeEnum.DEPT.getKey() == serviceRoleDataAuthDetailDTO.getDetailType()) {
                        //4.2.3selectFalg为true时查询组织-科室信息
                        getDeptDetial(dataAuthDetailDTO);
                    }
                    dataAuthDetailDTOS.add(dataAuthDetailDTO);
                });
                sysServiceDTO.setDataAuthDetailDTOS(dataAuthDetailDTOS);

                //5.添加角色列表
                List<RoleDTO> roleDTOS = new ArrayList<>();
                Map<Long, String> roleMap = new HashMap<>();
                serviceRoleDataAuthDetailDTOS.stream().forEach(serviceRoleDataAuthDetailDTO -> {
                    roleMap.put(serviceRoleDataAuthDetailDTO.getRoleId(), serviceRoleDataAuthDetailDTO.getRoleName());
                });
                for (Long roleID : roleMap.keySet()) {
                    RoleDTO role = new RoleDTO();
                    role.setId(roleID);
                    role.setName(roleMap.get(roleID));
                    roleDTOS.add(role);
                }
                sysServiceDTO.setRoles(roleDTOS);
                sysServiceDTOS.add(sysServiceDTO);
            });
            dataAuthDTO.setSysServiceDTOS(sysServiceDTOS);
        } else {
            Asserts.fail("请检查角色id或数据权限id");
        }
        return dataAuthDTO;
    }

    /**
     * @param dataAuthDetailDTO
     * @Description 获取组织-科室信息以及自定义科室
     * @Return void
     */
    public void getDeptDetial(DataAuthDetailDTO dataAuthDetailDTO) {
        //1.获取医院信息
        HospitalDTO hospitalDTO = new HospitalDTO();
        Hospital hospital = hospitalFacade.list(new QueryWrapper<Hospital>()
                .eq("id", SysUserUtils.getCurrentHospitalId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())).get(0);
        BeanUtils.copyProperties(hospital, hospitalDTO);
        //获取当前登录用户的组织信息
        List<HospitalDTO> hospitalDTOS = getHospitalInfo(hospitalDTO);
        getAllHospitalDTO(hospitalDTOS);
        //2.获取科室信息
        getDeptInfo(hospitalDTO, dataAuthDetailDTO);
        dataAuthDetailDTO.setHospitalDTOS(hospitalDTO);


    }

    /**
     * @param hospitalDTO
     * @param dataAuthDetailDTO
     * @Description取对应医院的科室信息
     * @Return void
     */
    private void getDeptInfo(HospitalDTO hospitalDTO, DataAuthDetailDTO dataAuthDetailDTO) {

        if (hospitalDTO != null) {
            if (ListUtil.isNotEmpty(hospitalDTO.getHospitalDTOS())) {
                for (HospitalDTO oneHospitalDTO : hospitalDTO.getHospitalDTOS()) {
                    //虚拟医院继续递归遍历到真实医院
                    getDeptInfo(oneHospitalDTO, dataAuthDetailDTO);
                }
                //真实医院：获取科室
            } else if ((HospitalTypeEnum.SINGLE_HOSPITAL.getKey() + "").equals(hospitalDTO.getType())) {
                List<Dept> depts = deptFacade.list(new QueryWrapper<Dept>()
                        .eq("hospital_id", hospitalDTO.getId())
                        .eq("is_deleted", IsDeleteEnum.N.getKey()));
                List<DeptDTO> deptDTOS = new ArrayList<>();
                if (ListUtil.isNotEmpty(depts)) {
                    for (Dept dept : depts) {
                        DeptDTO deptDTO = new DeptDTO();
                        BeanUtils.copyProperties(dept, deptDTO);
                        deptDTOS.add(deptDTO);
                        //该医院和该医院的科室 自定义关联关系为true
                        if (dataAuthDetailDTO.getDetailId() == deptDTO.getId()) {
                            hospitalDTO.setRelation(RelationEnum.Y.getName());
                            deptDTO.setRelation(RelationEnum.Y.getName());
                        }
                    }
                }
                hospitalDTO.setDepts(deptDTOS);
            }
        }
    }

    /**
     * @param hospitalDTOS
     * @Description 获取医院对应的子医院信息
     * @Return void
     */
    private void getAllHospitalDTO(List<HospitalDTO> hospitalDTOS) {
        if (ListUtil.isNotEmpty(hospitalDTOS)) {
            for (HospitalDTO tempHospitalDTO : hospitalDTOS) {
                getAllHospitalDTO(getHospitalInfo(tempHospitalDTO));
            }
        }
    }

    /**
     * @param hospitalDTO
     * @Description获取医院信息
     * @Return java.util.List<com.lantone.common.dto.HospitalDTO>
     */
    private List<HospitalDTO> getHospitalInfo(HospitalDTO hospitalDTO) {
        List<HospitalDTO> hospitalDTOS = new ArrayList<>();
        if (hospitalDTO.getId() != null) {
            List<Hospital> hospitals = hospitalFacade.list(new QueryWrapper<Hospital>()
                    .eq("parent_id", hospitalDTO.getId())
                    .eq("is_deleted", IsDeleteEnum.N.getKey()));
            hospitals.stream().forEach(hospital -> {
                HospitalDTO tempHospitalDTO = new HospitalDTO();
                BeanUtils.copyProperties(hospital, tempHospitalDTO);
                hospitalDTOS.add(tempHospitalDTO);
            });
            hospitalDTO.setHospitalDTOS(hospitalDTOS);
        }
        return hospitalDTOS;
    }

    /**
     * @param getDoctorPageVO
     * @Description通过医院id或科室id获取医生列表
     * @Return com.baomidou.mybatisplus.core.metadata.IPage<com.lantone.common.dto.DoctorDTO>
     */
    public IPage<GetDoctorPageDTO> getDoctorPage(GetDoctorPageVO getDoctorPageVO) {
        return doctorFacade.getBaseMapper().getDoctorPage(getDoctorPageVO);
    }

    /**
     * @param getDataAuthPageVO
     * @Description获取数据权限列表
     * @Return com.baomidou.mybatisplus.core.metadata.IPage<com.lantone.common.dto.DataAuthDTO>
     */
    public IPage<GetDataAuthPageDTO> getDataAuthPage(GetDataAuthPageVO getDataAuthPageVO) {
        //数据权限的创建者不一定是当前登录用户，创建者应为当前登录用户所在医院所有用户
        getDataAuthPageVO.setCreator(SysUserUtils.getCurrentPrincipleId());
        return dataAuthFacade.getBaseMapper().getDataAuthPage(getDataAuthPageVO);
    }
}
