package com.lantone.security.facade;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.additional.query.impl.LambdaQueryChainWrapper;
import com.lantone.common.enums.IsDeleteEnum;
import com.lantone.common.exception.ApiException;
import com.lantone.common.util.StringUtil;
import com.lantone.common.vo.GetHospitalSetVO;
import com.lantone.common.vo.SaveHospitalSetVO;
import com.lantone.dblayermbg.entity.HospitalSet;
import com.lantone.dblayermbg.facade.HospitalSetFacade;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author: cy
 * @time: 2021/7/30 16:30
 */
@Component
public class HospitalSetManagementFacade {
    @Autowired
    private HospitalSetFacade hospitalSetFacade;

    /**
     * @Description:  新增/修改参数
     * @Param: [saveHospitalSetVO]
     * @return: java.lang.Object
     * @Author: cy
     * @Date: 2021/7/30
     */
    public Object saveHospitalSet(SaveHospitalSetVO saveHospitalSetVO) {
        Boolean flag = false;
        //新增-校验重复数据
        checkHospitalSet(saveHospitalSetVO);
        HospitalSet hospitalSet = new HospitalSet();
        BeanUtils.copyProperties(saveHospitalSetVO, hospitalSet);
        if (null == saveHospitalSetVO.getId()) {
            flag = addHospitalSet(hospitalSet);
        } else if (null != saveHospitalSetVO.getId()) {
            flag = updateHospitalSet(hospitalSet);
        }
        return flag;
    }

    /**
     * @Description:  查看参数管理
     * @Param: [getHospitalSetVO]
     * @return: IPage<com.lantone.dblayermbg.entity.HospitalSet>
     * @Author: cy
     * @Date: 2021/7/30
     */
    public IPage<HospitalSet> getHospitalSet(GetHospitalSetVO getHospitalSetVO) {
        IPage<HospitalSet> pageInfo = new Page();
        LambdaQueryChainWrapper<HospitalSet> hospitalSetLambdaQuery = hospitalSetFacade.lambdaQuery();
        hospitalSetLambdaQuery.eq(HospitalSet::getIsDeleted, IsDeleteEnum.N.getKey());
        if (StringUtil.isNotEmpty(getHospitalSetVO.getHospitalName())) {
            hospitalSetLambdaQuery.like(HospitalSet::getHospitalName, getHospitalSetVO.getHospitalName());
        }
        if (StringUtil.isNotEmpty(getHospitalSetVO.getName())) {
            hospitalSetLambdaQuery.like(HospitalSet::getName, getHospitalSetVO.getName());
        }
        if (null != getHospitalSetVO.getCode()) {
            hospitalSetLambdaQuery.like(HospitalSet::getCode, getHospitalSetVO.getCode());
        }
        hospitalSetLambdaQuery.orderByDesc(HospitalSet::getGmtCreate);
        IPage<HospitalSet> page = hospitalSetLambdaQuery.page(pageInfo);
        return page;
    }

    /**
     * @Description: 删除参数管理-逻辑删除
     * @Param: [id]
     * @return: java.lang.Boolean
     * @Author: cy
     * @Date: 2021/7/30
     */
    public Boolean deleteHospitalSet(Integer id) {
        return hospitalSetFacade.lambdaUpdate().eq(HospitalSet::getId,id)
                .set(HospitalSet::getIsDeleted, IsDeleteEnum.Y.getKey()).update();
    }




    /**
     * @Description: 新增-校验重复数据
     * @Param: [saveHospitalSetVO]
     * @return: void
     * @Author: cy
     * @Date: 2021/7/30
     */
    public void checkHospitalSet(SaveHospitalSetVO saveHospitalSetVO) {
        if (null == saveHospitalSetVO.getId()) {
            Integer count = hospitalSetFacade.lambdaQuery()
                    .eq(HospitalSet::getIsDeleted, IsDeleteEnum.N.getKey())
                    .eq(HospitalSet::getName, saveHospitalSetVO.getName())
                    .eq(HospitalSet::getHospitalId, saveHospitalSetVO.getHospitalId())
                    .eq(HospitalSet::getVal, saveHospitalSetVO.getVal())
                    .count();
            if (count > 0) {
                throw new ApiException("当前参数添加重复");
            }
        }
    }

    /**
     * @Description:   新增参数
     * @Param: [hospitalSet]
     * @return: java.lang.Boolean
     * @Author: cy
     * @Date: 2021/7/30
     */
    public Boolean addHospitalSet(HospitalSet hospitalSet) {
        hospitalSet.setGmtCreate(new Date());
        hospitalSet.setCreator("程瑶-后期改");
        return hospitalSetFacade.save(hospitalSet);
    }

    /**
     * @Description: 修改参数
     * @Param: [hospitalSet]
     * @return: java.lang.Boolean
     * @Author: cy
     * @Date: 2021/7/30
     */
    public Boolean updateHospitalSet(HospitalSet hospitalSet) {
        hospitalSet.setGmtModified(new Date());
        hospitalSet.setModifier("程瑶改");
        return hospitalSetFacade.updateById(hospitalSet);
    }

    /**
     * @Description:  查看参数管理所属组织
     * @Param: [hospitalName]
     * @return: java.util.List<java.lang.String>
     * @Author: cy
     * @Date: 2021/8/9
     */
    public List<String> getHospitalNames(String hospitalName) {
        LambdaQueryChainWrapper<HospitalSet> hospitalSetLambdaQuery = hospitalSetFacade.lambdaQuery();
        if(StringUtil.isNotBlank(hospitalName)){
            hospitalSetLambdaQuery.like(HospitalSet::getHospitalName,hospitalName);
        }
        hospitalSetLambdaQuery.eq(HospitalSet::getIsDeleted, IsDeleteEnum.N.getKey());
        hospitalSetLambdaQuery.groupBy(HospitalSet::getHospitalName);
        return  hospitalSetLambdaQuery.list().stream().filter(obj -> StringUtil.isNotBlank(obj.getHospitalName())).map(obj -> obj.getHospitalName()).sorted().collect(Collectors.toList());
    }
}
