package com.lantone.security.facade;

import cn.hutool.crypto.digest.BCrypt;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.api.CommonResult;
import com.lantone.common.constant.AuthConstant;
import com.lantone.common.domain.UserDto;
import com.lantone.common.dto.GetDeptTreeDTO;
import com.lantone.common.dto.GetHospitalTreeDTO;
import com.lantone.common.dto.GetUserDTO;
import com.lantone.common.dto.GetUserHospitalsDTO;
import com.lantone.common.dto.GetUserPageDTO;
import com.lantone.common.dto.SoftwareDTO;
import com.lantone.common.dto.UserSoftwareHospitalDTO;
import com.lantone.common.enums.IsDeleteEnum;
import com.lantone.common.enums.StatusEnum;
import com.lantone.common.exception.Asserts;
import com.lantone.common.util.DateUtil;
import com.lantone.common.util.EntityUtil;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.SysUserUtils;
import com.lantone.common.vo.AddHospitalTreeVO;
import com.lantone.common.vo.AddUserVO;
import com.lantone.common.vo.GetUserPageVO;
import com.lantone.common.vo.LoginVO;
import com.lantone.common.vo.UpdateUserVO;
import com.lantone.dblayermbg.entity.DeptUser;
import com.lantone.dblayermbg.entity.Hospital;
import com.lantone.dblayermbg.entity.HospitalUser;
import com.lantone.dblayermbg.entity.Role;
import com.lantone.dblayermbg.entity.User;
import com.lantone.dblayermbg.entity.UserRole;
import com.lantone.dblayermbg.facade.DeptFacade;
import com.lantone.dblayermbg.facade.DeptUserFacade;
import com.lantone.dblayermbg.facade.HospitalFacade;
import com.lantone.dblayermbg.facade.HospitalUserFacade;
import com.lantone.dblayermbg.facade.RoleFacade;
import com.lantone.dblayermbg.facade.UserFacade;
import com.lantone.dblayermbg.facade.UserRoleFacade;
import com.lantone.security.enums.HospitalTypeEnum;
import com.lantone.security.enums.RelationEnum;
import com.lantone.security.service.AuthService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * @Description: 用户管理-业务处理类
 * @author: rengb
 * @time: 2021/7/20 12:39
 */
@Component
public class UserManagementFacade {

    @Autowired
    private AuthService authService;
    @Autowired
    private HospitalFacade hospitalFacade;
    @Autowired
    private HospitalUserFacade hospitalUserFacade;
    @Autowired
    private DeptUserFacade deptUserFacade;
    @Autowired
    private DeptFacade deptFacade;
    @Autowired
    private UserFacade userFacade;
    @Autowired
    private DeptUserFacade userDeptFacade;
    @Autowired
    private UserRoleFacade userRoleFacade;
    @Autowired
    private RoleFacade roleFacade;

    public CommonResult login(LoginVO loginVO) {
        Map<String, String> params = new HashMap<>();
        params.put("client_id", AuthConstant.SECURITY_CENTER_CLIENT_ID);
        params.put("client_secret", "lantone_20210723");
        params.put("grant_type", "password");
        params.put("username", loginVO.getUsername());
        params.put("password", loginVO.getPassword());
        CommonResult restResult = authService.getAccessToken(params);
        return restResult;
    }

    public UserDto loadUserByUsername(String username) {
        QueryWrapper<User> userQueryWrapper = new QueryWrapper<>();
        userQueryWrapper.eq("username", username);
        userQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey());
        List<User> userList = userFacade.list(userQueryWrapper);
        if (userList.size() == 0) {
            return null;
        }
        if (userList.size() > 1) {
            Asserts.fail("当前账号异常！");
        }

        UserDto userDTO = new UserDto();
        User user = userList.get(0);
        QueryWrapper<UserRole> userRoleQueryWrapper = new QueryWrapper<>();
        userRoleQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey());
        userRoleQueryWrapper.eq("user_id", user.getId());
        List<Long> roleIdList = userRoleFacade.list(userRoleQueryWrapper)
                .stream()
                .map(i -> i.getRoleId())
                .collect(Collectors.toList());

        QueryWrapper<Role> roleQueryWrapper = new QueryWrapper<>();
        roleQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey());
        roleQueryWrapper.eq("status", StatusEnum.Enable.getKey());
        roleQueryWrapper.in(ListUtil.isNotEmpty(roleIdList), "id", roleIdList);
        List<String> roleIdNames = roleFacade.list(roleQueryWrapper).stream().map(i -> i.getId() + "_" + i.getName()).collect(Collectors.toList());

        BeanUtils.copyProperties(user, userDTO);
        userDTO.setUserName(user.getUsername());
        userDTO.setRoles(roleIdNames);
        return userDTO;
    }

    /**
     * @param addUserVO
     * @Description添加用户
     * @Return boolean
     */
    public boolean addUser(AddUserVO addUserVO) {
        //1.新增用户基本信息
        User user = new User();
        BeanUtils.copyProperties(addUserVO, user);
        user.setCreator(SysUserUtils.getCurrentPrincipleId()+"");
        user.setGmtCreate(DateUtil.now());
        user.setPassword(BCrypt.hashpw(user.getPassword()));
        //1.1校验用户
        checkUser(user.getUsername(), addUserVO.getAddHospitalTreeVO().getHospitals(), addUserVO.getRoles());
        try {
            if (userFacade.save(user)) {
                //2.插入用户所属组织
                return saveUserHospital(addUserVO.getAddHospitalTreeVO(), user.getId(), addUserVO.getRoles());
            } else {
                Asserts.fail("用户添加失败");
            }
        } catch (Exception e) {
            Asserts.fail("用户添加失败");
        }
        return false;
    }

    /**
     * @param usreName
     * @param hospitals 插入用户组织id
     * @param roles 插入用户角色id
     * @Description用户校验
     * @Return void
     */
    private void checkUser(String usreName, List<Long> hospitals, List<Long> roles) {

        //1.当前登录用户、组织是否还存在、能用
        loginCheck();
        //2.当前用户是否已存在
        if (userFacade.list(new QueryWrapper<User>()
                .eq("username", usreName)
                .eq("is_deleted", IsDeleteEnum.N.getKey())).size() > 0) {
            Asserts.fail("该用户已存在,请更换用户名");
        }
        //3.校验插入用户所属组织是否正常
        checkHospitals(hospitals);
        //4.校验插入用户角色是否正常
        checkRoles(roles);
    }

    /**
     * @param roles
     * @Description插入角色校验
     * @Return void
     */
    private void checkRoles(List<Long> roles) {
        if (ListUtil.isNotEmpty(roles)) {
            if (roles.size() != roleFacade.list(new QueryWrapper<Role>()
                    .in("id", roles)
                    .eq("status", StatusEnum.Enable.getKey())
                    .eq("is_deleted", IsDeleteEnum.N.getKey())).size()) {
                Asserts.fail("插入用户角色列表有改动,请刷新页面重新操作");
            }
        }
    }

    /**
     * @param hospitals
     * @Description插入组织校验
     * @Return void
     */
    private void checkHospitals(List<Long> hospitals) {
        if (ListUtil.isNotEmpty(hospitals)) {
            if (hospitals.size() != hospitalFacade.list(new QueryWrapper<Hospital>()
                    .in("id", hospitals)
                    .eq("status", StatusEnum.Enable.getKey())
                    .eq("is_deleted", IsDeleteEnum.N.getKey())).size()) {
                Asserts.fail("插入用户所属组织有改动,请刷新页面重新操作");
            }
        }
    }

    /**
     * @param
     * @Description操作用户校验
     * @Return void
     */
    private void loginCheck() {
        if (hospitalFacade.list(new QueryWrapper<Hospital>()
                .eq("id", SysUserUtils.getCurrentHospitalId())
                .eq("status", StatusEnum.Enable.getKey())
                .eq("is_deleted", IsDeleteEnum.N.getKey())).size() <= 0) {
            Asserts.fail("操作用户所属组织已被禁用或删除");
        }
        if (userFacade.list(new QueryWrapper<User>()
                .eq("id", SysUserUtils.getCurrentPrincipleId())
                .eq("status", StatusEnum.Enable.getKey())
                .eq("is_deleted", IsDeleteEnum.N.getKey())).size() <= 0) {
            Asserts.fail("该账号暂未开通或已被禁用，请联系管理员~");
        }
    }

    /**
     * @param addHospitalTreeVO
     * @param userId
     * @Description保存用户关联的医院、科室
     * @Return boolean
     */
    private boolean saveUserHospital(AddHospitalTreeVO addHospitalTreeVO, Long userId, List<Long> roles) {

        if (ListUtil.isEmpty(addHospitalTreeVO.getDepts()) && ListUtil.isEmpty(addHospitalTreeVO.getHospitals())) {
            Asserts.fail("用户关联的医院或科室都为空");
        }
        //插入用户和科室的关系（没有科室信息默认为全院）
        if (ListUtil.isNotEmpty(addHospitalTreeVO.getDepts())) {
            addHospitalTreeVO.getDepts().stream().forEach(deptID -> {
                DeptUser userDept = new DeptUser();
                userDept.setDeptId(deptID);
                userDept.setUserId(userId);
                userDeptFacade.save(userDept);
            });
        }
        //插入用户和医院的关系
        if (ListUtil.isNotEmpty(addHospitalTreeVO.getHospitals())) {
            addHospitalTreeVO.getHospitals().stream().forEach(hospitalId -> {
                HospitalUser hospitalUser = new HospitalUser();
                hospitalUser.setUserId(userId);
                hospitalUser.setHospitalId(hospitalId);
                hospitalUserFacade.save(hospitalUser);
            });
        }
        //插入用户和角色的关系
        return saveUserRole(userId, roles);
    }

    /**
     * @param userId
     * @param roles
     * @Description插入用户和角色的关系
     * @Return boolean
     */
    private boolean saveUserRole(Long userId, List<Long> roles) {
        if (ListUtil.isNotEmpty(roles)) {
            roles.stream().forEach(roleId -> {
                UserRole userRole = new UserRole();
                userRole.setRoleId(roleId);
                userRole.setUserId(userId);
                userRoleFacade.save(userRole);
            });
            return true;
        }
        return false;
    }

    /**
     * @param
     * @Description获取当前用户的组织机构树
     * @Return java.util.List<com.lantone.common.dto.GetHospitalTreeDTO>
     */
    public List<GetHospitalTreeDTO> getLoginUserHospitalTree() {
        Hospital hospital = hospitalFacade.getById(SysUserUtils.getCurrentHospitalId());
        //获取当前医院的树状结构
        List<GetHospitalTreeDTO> hospitalTrees = hospitalFacade.getBaseMapper()
                .getHospitalTreeInfo(SysUserUtils.getCurrentHospitalId(), StatusEnum.Enable.getKey());
        //根据parentid分组
        Map<Long, List<GetHospitalTreeDTO>> hospitalMap = EntityUtil.makeEntityListMap(hospitalTrees, "parentId");
        //深度遍历拼装结果
        List<GetHospitalTreeDTO> hospitalTree = hospitalMap.get(hospital.getParentId());
        for (GetHospitalTreeDTO getHospitalTreeDTO : hospitalTree) {
            //真实医院添加科室信息
            if (HospitalTypeEnum.SINGLE_HOSPITAL.getKey() == Integer.parseInt(getHospitalTreeDTO.getType())) {
                getHospitalTreeDTO.setDepts(deptFacade.getBaseMapper().getDeptInfo(getHospitalTreeDTO.getHospitalId()));
            }
            getSonHospital(getHospitalTreeDTO, hospitalMap);
        }
        return hospitalTree;
    }


    /**
     * @param updateUserVO
     * @Description修改用户
     * @Return boolean
     */
    public boolean updateUser(UpdateUserVO updateUserVO) {
        //1.删除该用户关联的科室、医院
        //1.1校验用户
        checkUser(updateUserVO.getUsername(), updateUserVO.getAddHospitalTreeVO().getHospitals(), updateUserVO.getRoles());
        if (delUserHospitalORDept(updateUserVO.getId())) {
            User user = new User();
            BeanUtils.copyProperties(updateUserVO, user);
            user.setPassword(BCrypt.hashpw(user.getPassword()));
            if (userFacade.updateById(user)) {
                //2.插入用户新关联的医院、科室
                return saveUserHospital(updateUserVO.getAddHospitalTreeVO(), updateUserVO.getId(), updateUserVO.getRoles());
            } else {
                Asserts.fail("用户基本信息修改失败");
            }
        }
        return false;
    }

    /**
     * @param id
     * @Description删除用户与医院、科室的关联关系
     * @Return boolean
     */
    private boolean delUserHospitalORDept(Long id) {
        if (hospitalUserFacade.remove(new UpdateWrapper<HospitalUser>()
                .eq("user_id", id)
                .eq("is_deleted", IsDeleteEnum.N.getKey()))) {
            if (userRoleFacade.remove(new UpdateWrapper<UserRole>()
                    .eq("user_id", id)
                    .eq("is_deleted", IsDeleteEnum.N.getKey()))) {
                if (userDeptFacade.list(new QueryWrapper<DeptUser>()
                        .eq("user_id", id)
                        .eq("is_deleted", IsDeleteEnum.N.getKey())).size() > 0) {
                    return userDeptFacade.remove(new UpdateWrapper<DeptUser>()
                            .eq("user_id", id)
                            .eq("is_deleted", IsDeleteEnum.N.getKey()));
                }
                return true;
            } else {
                Asserts.fail("用户和角色的关联关系删除失败");
            }
        } else {
            Asserts.fail("用户和医院的关联关系删除失败");
        }
        return false;
    }

    /**
     * @param userId
     * @Description删除用户
     * @Return boolean
     */
    public boolean deleteUser(Long userId) {
        return userFacade.update(new UpdateWrapper<User>()
                .set("is_deleted", IsDeleteEnum.Y.getKey())
                .eq("id", userId));
    }

    /**
     * @param userId
     * @param status
     * @Description启用禁用用户
     * @Return boolean
     */
    public boolean disableUser(Long userId, String status) {

        return userFacade.update(new UpdateWrapper<User>()
                .set("status", status)
                .eq("id", userId)
                .eq("is_deleted", IsDeleteEnum.N.getKey()));
    }

    /**
     * @param userId
     * @Description获取用户详情
     * @Return com.lantone.common.dto.GetUserDTO
     */
    public GetUserDTO getUserById(Long userId) {
        GetUserDTO getUserDTO = new GetUserDTO();
        //1.获取用户的基本信息
        BeanUtils.copyProperties(userFacade.getById(userId), getUserDTO);
        //2.获取当前用户的组织类型
        Hospital hospital = hospitalFacade.getById(SysUserUtils.getCurrentHospitalId());
        //3.获取查询用户的科室id列表、医院id列表
        List<Long> hospitals = hospitalUserFacade.list(new QueryWrapper<HospitalUser>()
                .eq("user_id", userId)
                .eq("is_deleted", IsDeleteEnum.N.getKey())).stream().map(HospitalUser::getHospitalId).collect(Collectors.toList());
        List<Long> depts = deptUserFacade.list(new QueryWrapper<DeptUser>()
                .eq("user_id", userId)
                .eq("is_deleted", IsDeleteEnum.N.getKey())).stream().map(DeptUser::getDeptId).collect(Collectors.toList());
        //3.2获取当前登录用户的组织机构树
        //获取当前医院的树状结构
        List<GetHospitalTreeDTO> hospitalTrees = hospitalFacade.getBaseMapper()
                .getHospitalTreeInfo(SysUserUtils.getCurrentHospitalId(), StatusEnum.Enable.getKey());
        //根据parentid分组
        Map<Long, List<GetHospitalTreeDTO>> hospitalMap = EntityUtil.makeEntityListMap(hospitalTrees, "parentId");
        //深度遍历拼装结果
        List<GetHospitalTreeDTO> hospitalTree = hospitalMap.get(hospital.getParentId());
        getHospitalDeptInfo(hospitalMap, hospitals, depts, hospitalTree);
        getUserDTO.setHospitals(hospitalTree);

        //4.获取用户的角色
        getUserDTO.setRoles(userRoleFacade.getBaseMapper().getUserRoles(userId));
        return getUserDTO;
    }


    /**
     * 递归获取医院结构
     *
     * @param hospital    当前医院
     * @param hospitalMap 医院集
     */
    public void getSonHospital(GetHospitalTreeDTO hospital, Map<Long, List<GetHospitalTreeDTO>> hospitalMap, List<Long> hospitals, List<Long> depts) {
        //通过查询用户所在的医院集添加关联关系
        if (hospitals.contains(hospital.getHospitalId())) {
            hospital.setRelation(RelationEnum.Y.getName());
        }
        List<GetHospitalTreeDTO> hospitalTreeDTOS = hospitalMap.get(hospital.getHospitalId());
        if (ListUtil.isNotEmpty(hospitalTreeDTOS)) {
            hospital.setChildren(hospitalTreeDTOS);
            getHospitalDeptInfo(hospitalMap, hospitals, depts, hospitalTreeDTOS);
        }
    }

    /**
     * @param hospitalMap      医院结果级
     * @param hospitals        查询用户所在医院集
     * @param depts            查询用户所在科室集
     * @param hospitalTreeDTOS 当前遍历医院集
     * @Description
     * @Return void
     */
    private void getHospitalDeptInfo(Map<Long, List<GetHospitalTreeDTO>> hospitalMap, List<Long> hospitals, List<Long> depts, List<GetHospitalTreeDTO> hospitalTreeDTOS) {

        for (GetHospitalTreeDTO bean : hospitalTreeDTOS) {
            //真实医院添加科室信息
            if (HospitalTypeEnum.SINGLE_HOSPITAL.getKey() == Integer.parseInt(bean.getType())) {
                List<GetDeptTreeDTO> getDeptTreeDTOList = deptFacade.getBaseMapper().getDeptInfo(bean.getHospitalId());
                //通过查询用户所在的科室集添加关联关系
                getDeptTreeDTOList.stream().forEach(getDeptTreeDTO -> {
                    if (depts.contains(getDeptTreeDTO.getDeptId())) {
                        getDeptTreeDTO.setRelation(RelationEnum.Y.getName());
                    }
                });
                bean.setDepts(getDeptTreeDTOList);
            }
            getSonHospital(bean, hospitalMap, hospitals, depts);
        }
    }

    /**
     * 递归获取医院结构
     *
     * @param hospital    当前医院
     * @param hospitalMap 医院集
     * @return 空
     */
    public void getSonHospital(GetUserHospitalsDTO hospital, Map<Long, List<GetUserHospitalsDTO>> hospitalMap) {
        List<GetUserHospitalsDTO> hospitals = hospitalMap.get(hospital.getHospitalId());
        if (ListUtil.isNotEmpty(hospitals)) {
            hospital.setChildren(hospitals);
            for (GetUserHospitalsDTO bean : hospitals) {
                getSonHospital(bean, hospitalMap);
            }
        }
    }

    /**
     * 递归获取医院结构
     *
     * @param hospital    当前医院
     * @param hospitalMap 医院集
     * @return 空
     */
    public void getSonHospital(GetHospitalTreeDTO hospital, Map<Long, List<GetHospitalTreeDTO>> hospitalMap) {
        List<GetHospitalTreeDTO> hospitals = hospitalMap.get(hospital.getHospitalId());
        if (ListUtil.isNotEmpty(hospitals)) {
            hospital.setChildren(hospitals);
            for (GetHospitalTreeDTO bean : hospitals) {
                //真实医院添加科室信息
                if (HospitalTypeEnum.SINGLE_HOSPITAL.getKey() == Integer.parseInt(bean.getType())) {
                    bean.setDepts(deptFacade.getBaseMapper().getDeptInfo(bean.getHospitalId()));
                }
                getSonHospital(bean, hospitalMap);
            }
        }
    }

    /**
     * @param getUserPageVO
     * @Description获取登录用户的用户列表
     * @Return com.baomidou.mybatisplus.core.metadata.IPage<com.lantone.common.dto.GetUserPageDTO>
     */
    public IPage<GetUserPageDTO> getUserPage(GetUserPageVO getUserPageVO) {

        getUserPageVO.setHospitalId(SysUserUtils.getCurrentHospitalId());
        //查出的用户列表不能包含当前登录的管理用户
        getUserPageVO.setLocalUserID(SysUserUtils.getCurrentPrincipleId());
        return hospitalUserFacade.getBaseMapper().getUserPage(getUserPageVO);
    }

    /**
     * @param
     * @Description获取用户所属组织
     * @Return java.util.List<com.lantone.common.dto.GetHospitalTreeDTO>
     */
    public Map<String, Object> getUserHospitals() {
        Map<String, Object> out = new HashMap<>();
        //但系统单家医院直接跳转
        boolean jumpFlag = true;
        //1.获取用户系统信息
        List<SoftwareDTO> softwares = userFacade.getBaseMapper().getUserSoftwareByUId(SysUserUtils.getCurrentPrincipleId());
        if (softwares.size() > 1) {
            jumpFlag = false;
        }
        AtomicInteger count = new AtomicInteger();
        List<UserSoftwareHospitalDTO> userSoftwareHospitals = new ArrayList<>();
        if (ListUtil.isNotEmpty(softwares)) {
            softwares.stream().forEach(softwareDTO -> {
                UserSoftwareHospitalDTO tempU = new UserSoftwareHospitalDTO();
                BeanUtils.copyProperties(softwareDTO, tempU);
                //获取该系统下的医院
                List<GetUserHospitalsDTO> softwareHospitals = userFacade.getBaseMapper()
                        .getUserSoftwareHospitals(SysUserUtils.getCurrentPrincipleId(), softwareDTO.getId());
                count.set(count.get() + softwareHospitals.size());
                //转换成树状结构
                Map<Long, List<GetUserHospitalsDTO>> softwareHospitalMap = EntityUtil.makeEntityListMap(softwareHospitals, "parentId");
                tempU.setHospitals(transTreeData(softwareHospitalMap, softwareHospitals));
                userSoftwareHospitals.add(tempU);
            });
        }
        if (count.get() > 1) {
            jumpFlag = false;
        }
        out.put("jump", jumpFlag);
        out.put("software", userSoftwareHospitals);
        return out;
    }

    /**
     * @param softwareHospitalMap
     * @param softwareHospitals
     * @Description将医院信息转换成树状结构
     * @Return java.util.List<com.lantone.common.dto.GetUserHospitalsDTO>
     */
    private List<GetUserHospitalsDTO> transTreeData(Map<Long, List<GetUserHospitalsDTO>> softwareHospitalMap, List<GetUserHospitalsDTO> softwareHospitals) {
        if (softwareHospitalMap == null) return null;
        List<GetUserHospitalsDTO> out = new ArrayList<>();
        Set<Long> tempParents = new HashSet<>();
        Set<Long> parents = softwareHospitalMap.keySet();
        parents.stream().forEach(id -> {
            softwareHospitals.stream().forEach(getUserHospitalsDTO -> {
                if (getUserHospitalsDTO.getHospitalId().equals(id)) {
                    tempParents.add(id);
                }
            });
        });
        parents.stream().forEach(parentId -> {
            if (!tempParents.contains(parentId)) {
                List<GetUserHospitalsDTO> hospitalListInfos = softwareHospitalMap.get(parentId);
                for (GetUserHospitalsDTO bean : hospitalListInfos) {
                    getSonHospital(bean, softwareHospitalMap);
                }
                out.addAll(hospitalListInfos);
            }
        });
        return out;
    }
}