package com.lantone.security.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.api.CommonResult;
import com.lantone.common.vo.GetDictionaryVO;
import com.lantone.common.vo.SaveDictionaryVO;
import com.lantone.dblayermbg.entity.DictionaryInfo;
import com.lantone.security.facade.DictionaryManagementFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description: 字典管理API
 * @author: cy
 * @time: 2021/7/20 12:39
 */
@RestController
@Api(value = "字典管理API", tags = { "字典管理API" })
@RequestMapping("/dictionaryManage")
public class DictionaryManagementController {
    @Autowired
    private DictionaryManagementFacade dictionaryManagementFacade;

    @ApiOperation(value = "新增/修改字典[by:cy]",
            notes = "id:新增操作为null<br>" +
                    "groupType:代码类别，必填<br>" +
                    "val:字典编码，必填<br>" +
                    "name:代码名称，必填<br>" +
                    "remark:字典说明，必填<br>" +
                    "status:是否启用，必填<br>")
    @PostMapping("/saveDictionary")
    @Transactional
    public CommonResult<Object> saveDictionary(@RequestBody @Valid SaveDictionaryVO saveDictionaryVO) {
        return CommonResult.success(dictionaryManagementFacade.saveDictionary(saveDictionaryVO));
    }

    @ApiOperation(value = "查看字典管理[by:cy]",
            notes = "name:代码名称，<br>" +
                    "groupType:代码类别，<br>")
    @PostMapping("/getDictionary")
    public CommonResult<IPage<DictionaryInfo>> getDictionary(@RequestBody GetDictionaryVO getDictionaryVO) {
        return CommonResult.success(dictionaryManagementFacade.getDictionary(getDictionaryVO));
    }

    @ApiOperation(value = "删除字典管理[by:cy]",
            notes = "id:字典id，必填<br>")
    @PostMapping("/deleteDictionary")
    @Transactional
    public CommonResult<Boolean> deleteDictionary(@RequestParam Integer id) {
        return CommonResult.success(dictionaryManagementFacade.deleteDictionary(id));
    }

    @ApiOperation(value = "查看字典管理代码类别[by:cy]",
            notes ="groupType:代码类别<br>")
    @PostMapping("/getCodeCategory")
    public CommonResult<List<String>> getCodeCategory(@RequestParam("groupType")String groupType) {
        return CommonResult.success(dictionaryManagementFacade.getCodeCategory(groupType));
    }
}
