package com.lantone.security.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.api.CommonResult;
import com.lantone.common.vo.GetHospitalSetVO;
import com.lantone.common.vo.SaveHospitalSetVO;
import com.lantone.dblayermbg.entity.HospitalSet;
import com.lantone.security.facade.HospitalSetManagementFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description: 参数管理API
 * @author: cy
 * @time: 2021/7/20 12:39
 */
@RestController
@Api(value = "参数管理API", tags = { "参数管理API" })
@RequestMapping("/hospitalSetManage")
public class HospitalSetManagementController {
    @Autowired
    private HospitalSetManagementFacade hospitalSetManagementFacade;

    @ApiOperation(value = "新增/修改参数[by:cy]",
            notes = "id:新增操作为null<br>" +
                    "hospitalId:医院ID<br>" +
                    "val:参数值，必填<br>" +
                    "code:参数名，必填<br>" +
                    "remark:参数描述，必填<br>" +
                    "name:参数说明，必填<br>")
    @PostMapping("/saveHospitalSet")
    @Transactional
    public CommonResult<Object> saveHospitalSet(@RequestBody @Valid SaveHospitalSetVO saveHospitalSetVO) {
        return CommonResult.success(hospitalSetManagementFacade.saveHospitalSet(saveHospitalSetVO));
    }

    @ApiOperation(value = "查看参数管理[by:cy]",
            notes = "name:代码名称，<br>" +
                    "groupType:代码类别，<br>")
    @PostMapping("/getHospitalSet")
    public CommonResult<IPage<HospitalSet>> getDictionary(@RequestBody GetHospitalSetVO getHospitalSetVO) {
        return CommonResult.success(hospitalSetManagementFacade.getHospitalSet(getHospitalSetVO));
    }

    @ApiOperation(value = "删除参数管理[by:cy]",
            notes = "id:参数id，必填<br>")
    @PostMapping("/deleteHospitalSet")
    @Transactional
    public CommonResult<Boolean> deleteHospitalSet(@RequestParam("id") Integer id) {
        return CommonResult.success(hospitalSetManagementFacade.deleteHospitalSet(id));
    }

    @ApiOperation(value = "查看参数管理所属组织[by:cy]",
            notes = "hospitalName:所属组织<br>")
    @PostMapping("/getHospitalNames")
    public CommonResult<List<String>> getHospitalNames(@RequestParam("hospitalName") String hospitalName) {
        return CommonResult.success(hospitalSetManagementFacade.getHospitalNames(hospitalName));
    }
}
