package com.lantone.security.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.api.CommonResult;
import com.lantone.common.dto.DeptListDTO;
import com.lantone.common.dto.ViewRegionDTO;
import com.lantone.common.vo.CheckSolveRegionVO;
import com.lantone.common.vo.GetDeptListVO;
import com.lantone.common.vo.GetRegionVO;
import com.lantone.common.vo.SaveRegionVO;
import com.lantone.security.facade.RegionManagementFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description: 病区管理API
 * @author: cy
 * @time: 2021/7/20 12:39
 */
@RestController
@Api(value = "病区管理API", tags = { "病区管理API" })
@RequestMapping("/regionManage")
public class RegionManagementController {
    @Autowired
    private RegionManagementFacade sysRegionFacade;

    @ApiOperation(value = "新增/修改病区[by:cy]",
            notes = "hospitalName:上级组织名称，必填<br>" +
                    "id:新增操作为null <br>" +
                    "name:病区名称，必填<br>" +
                    "code:病区编码，<br>" +
                    "deptIds:关联科室，<br>" +
                    "station:区域类别，必填<br>" +
                    "orderNo:排序，<br>" +
                    "status:是否启用，<br>")
    @PostMapping("/saveRegion")
    @Transactional
    public CommonResult<Object> saveRegion(@RequestBody @Valid SaveRegionVO saveRegionVo) {
        return CommonResult.success(sysRegionFacade.saveRegion(saveRegionVo));
    }

    @ApiOperation(value = "校验/禁用病区管理[by:cy]",
            notes = "hospitalId:医院ID<br>" +
                    "name:病区名称<br>" +
                    "id:病区id，必填<br>" +
                    "mark:0-禁用校验标识 1-禁用此病区，必填<br>")
    @PostMapping("/checkSolveRegion")
    public CommonResult<Object> checkSolveRegion(@RequestBody @Valid CheckSolveRegionVO checkSolveRegionVO) {
        return CommonResult.success(sysRegionFacade.checkSolveRegion(checkSolveRegionVO));
    }


    @ApiOperation(value = "查看病区管理[by:cy]",
            notes = "hospitalId:医院ID<br>" +
                    "name:病区名称，<br>" +
                    "deptName:关联科室，<br>" +
                    "status:启用状态 0-否 1-是 <br>")
    @PostMapping("/getRegion")
    public CommonResult<IPage<ViewRegionDTO>> getRegion(@RequestBody GetRegionVO getRegionVO) {
        return CommonResult.success(sysRegionFacade.getWard(getRegionVO));
    }


    @ApiOperation(value = "删除病区管理[by:cy]",
            notes = "id:病区id，必填<br>")
    @PostMapping("/deleteRegion")
    @Transactional
    public CommonResult<Boolean> deleteRegion(@RequestParam Integer id) {
        return CommonResult.success(sysRegionFacade.deleteRegion(id));
    }

    @ApiOperation(value = "获取科室列表[by:cy]",
            notes = "标记（0-返回科室列表 1-返回互斥科室列表），必填<br>")
    @PostMapping("/getDeptList")
    public CommonResult<List<DeptListDTO>> getDeptList(@RequestBody GetDeptListVO getDeptListVO) {
        return CommonResult.success(sysRegionFacade.getDept(getDeptListVO));
    }
}
